#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing look using G'MIC Mc Pendraw plug-in

# (c) Dinasset 2021
#
#   History:
#
#   v1.0: 2021-03-31: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
wrkWH=3000

############################################################################
# Service routines
#---------------------------------------------------------------------------

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

####################
#   M A I N
#-------------------
    
def DrawStyleY (inImage, inDrawable, outimgWH, Mask,
                doFrame, doFlatten) :
    
    pdb.gimp_context_push()
    # source image characteristics
    width = inImage.width
    height = inImage.height
    OrigWH = width+height
    otype = inImage.base_type
    # image name
    iname = inImage.name
    split = iname.split("(", 1)
    iname = split[0]
    end = len(iname)-2
    iname = iname[1:end]    # remove []
    #info_msg (iname)
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    # working sizes
    Ratio = wrkWH*1.00/OrigWH*1.00
    rWidth = int(width*Ratio)
    rHeight = int(height*Ratio)
    rWH = rWidth+rHeight
    
    # START    
    # create output image
    newImage = pdb.gimp_image_new (rWidth, rHeight, otype )
    newBG = pdb.gimp_layer_new_from_drawable (inDrawable, newImage)
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale (newBG, rWidth, rHeight, False)
    newBG.name = inDrawable.name
    if '.' in newBG.name:
        split = newBG.name.split(".", 1)
        newBG.name = split[0]
    pdb.gimp_image_set_filename (newImage,iname)
    
    original = newBG.copy()
    newImage.add_layer (original, 1)
    original.name = "original"
    
    (meanV,B,C,D,E,F)=pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                      0.0, 1.0)
    #info_msg ("meanV="+str(meanV))
    if meanV<100.0:  # dark images
        while meanV<100.0:  # adjust dark images
            pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE,
                                            6, (0.0,0.0, meanV/255,(meanV+8)/255, 1.0,1.0))
            (meanV,B,C,D,E,F)=pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                      0.0, 1.0)
        #info_msg ("meanV adj="+str(meanV))
    elif meanV>155.0:  # dark images
        while meanV>155.0:  # adjust light images
            pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE,
                                             6, (0.0,0.0, meanV/255,(meanV-8)/255, 1.0,1.0))
            (meanV,B,C,D,E,F)=pdb.gimp_drawable_histogram (newBG, HISTOGRAM_VALUE,
                                                      0.0, 1.0)
        #info_msg ("meanV adj="+str(meanV))
    # remove whites    
    pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE,   
                                     6, (0.0,0.0, 0.5,0.45,  1.0,0.9))
    
    
# ==STEP1=================================================================================
    
    #info_msg("applying Mc Pendraw")   
    pdb.plug_in_gmic_qt(newImage, newBG, 1,2 ,
    #[G'MIC] Mc Pendraw: mc_pendraw 30,0.1,2000,256,1,1,0,0,255,0,0
    "-v - -mc_pendraw 30,0.1,2000,256,1,1,0,0,255,0,0 ")
    PenDraw = newImage.layers[0]
    PenDraw.name = "Telperion Mc Pendraw"
    PenDraw.opacity = 100.0
    PenDraw.mode = LAYER_MODE_NORMAL    #DIVIDE

    Step1Outcome = pdb.gimp_layer_new_from_visible(newImage,newImage,
                                                "step 1 Outcome")
    newImage.add_layer(Step1Outcome,0)
    Step1Outcome.mode=LAYER_MODE_NORMAL
    Step1Outcome.opacity = 100.0
    
# ==STEP2=================================================================================
    
    #info_msg("applying Unsharp")
    pdb.plug_in_unsharp_mask (newImage, Step1Outcome, 5.0, 1.0, 0)
    # BG on Top for Hue 
    OrigOnTop = original.copy()
    newImage.add_layer(    OrigOnTop,0)
    OrigOnTop.name = "Original on Top for Hue "
    OrigOnTop.mode = LAYER_MODE_LCH_HUE
    OrigOnTop.opacity = 100.0
     
    Step2Outcome = pdb.gimp_layer_new_from_visible(newImage,newImage,
                                                "step 2 Outcome")
    newImage.add_layer(Step2Outcome,0)
    Step2Outcome.mode=LAYER_MODE_NORMAL
    Step2Outcome.opacity = 100.0
    
# ==STEP3=================================================================================
     
    #iremoved
    
# ==FINAL STEP=================================================================================
     
    maskedBG = newBG.copy()
    newImage.add_layer(maskedBG,0)
    maskedBG.name = "BG on Top for values touch up "
    maskedBG.mode = LAYER_MODE_HSV_VALUE
    maskedBG.opacity = 75.0 # basic
    if Mask>0:
        maskedBG.opacity = 100.0  # when a user mask is applied                                                                    
        if Mask==1:
            blackmask = pdb.gimp_layer_create_mask (maskedBG, ADD_MASK_BLACK)    
            pdb.gimp_layer_add_mask (maskedBG, blackmask)
            optmask=", black mask"
        if Mask in (2,3):
            graymask = pdb.gimp_layer_create_mask (maskedBG, ADD_MASK_COPY)      
            pdb.gimp_layer_add_mask (maskedBG, graymask)
            if Mask==2: 
                optmask=", gray-copy mask"
            if Mask==3:
                pdb.plug_in_vinvert (newImage, graymask)
                optmask=", gray-copy inverted mask"
    else:   optmask=", basic smooth"
    # duplicate for using layer mode SATURATION
    maskedBG2 = maskedBG.copy()
    newImage.add_layer(maskedBG2,0)
    maskedBG2.name = "BG on Top for saturation touch up"
    maskedBG2.mode = LAYER_MODE_HSV_SATURATION
    
    Outcome = pdb.gimp_layer_new_from_visible(newImage,newImage, "Outcome")
    newImage.add_layer(Outcome,0)
    
    if doFlatten==True:
        pdb.gimp_item_set_visible (original, False) # temp
        Outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_IMAGE)
        pdb.gimp_item_set_visible (original, True) # reset
        
    strokeMode = "by G'mic Telperion plug-in"
    Outcome.name = st+" "+iname+" stroked "+strokeMode
      
    if doFrame == True:
        # frame
        frameSize = int(round(wrkWH/8))   
        if frameSize%2<>0: frameSize+=1
        fWidth = int(round(rWidth+frameSize*1.0))
        fHeight = int(round(rHeight+frameSize*1.0))
        pdb.gimp_image_resize (newImage, fWidth, fHeight,
                               int(round(frameSize*0.5)), int(round(frameSize*0.5)))
        FrameLayer = pdb.gimp_layer_new(newImage, fWidth, fHeight, RGB_IMAGE,
                        "Frame ", 100.0, LAYER_MODE_NORMAL)
        newImage.add_layer(FrameLayer, 0)
        pdb.gimp_context_set_pattern ("Walnut")
        pdb.gimp_drawable_fill (FrameLayer, FILL_PATTERN)
        pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
                                         frameSize/2, frameSize/2,
                                         rWidth, rHeight)
        pdb.gimp_layer_add_alpha (FrameLayer)
        pdb.gimp_edit_cut (FrameLayer)
        pdb.gimp_selection_border (newImage, frameSize/50)
        pdb.gimp_context_set_foreground ((207,181,59,255))  # old gold
        pdb.gimp_edit_fill (FrameLayer, FILL_FOREGROUND)
        pdb.gimp_selection_all (newImage)
        pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_SUBTRACT,
                                         frameSize/10, frameSize/10,
                                         fWidth-frameSize/5, fHeight-frameSize/5)
        pdb.gimp_context_set_foreground ((60,30,10,255))  # very dark gold
        pdb.gimp_edit_fill (FrameLayer, FILL_FOREGROUND)
        pdb.gimp_selection_none (newImage)
        
        #Outcome = pdb.gimp_image_merge_down (newImage, FrameLayer, EXPAND_AS_NECESSARY)
     
    if outimgWH==0: outimgWH=OrigWH
    if outimgWH<>wrkWH:
        uRatio = outimgWH*1.00/wrkWH*1.00
        uWidth = int(rWidth*uRatio)
        uHeight = int(rHeight*uRatio)
        #info_msg ("SCALING")   # not during tests, scaling is slow
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
        
    # tier down
    
    pdb.gimp_selection_none(newImage)
    pdb.gimp_context_pop()
    
    pdb.gimp_display_new(newImage)
     
    return 

# This is the plugin registration function

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/Drawing/"
copyrightYear="2021"
desc="Draw Effect Style Y "

register(
    "DrawStyleY",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+"\n (0=same as input)",+
                              3000, (0, 8000, 100)),
      (PF_OPTION, "maskbg", "Add Masked BG on Top?"+
                            "\n  (for adjustments)", 0, ["*none (basic smooth)",
                                                 "black (user paints over)",
                                                 "gray copy (to smooth light areas)",
                                                 "gray copy inverted (to smooth dark areas)",
                                                ]),
      (PF_TOGGLE, "frame", "Add a frame?", True),
      (PF_TOGGLE, "flatten", "Flatten image?", True),
    ], 
    [
      (PF_DRAWABLE, "drawable", "Output drawable", None),
    ],
    DrawStyleY,
    menu=menu
    )

main()
